from notebookutils import mssparkutils


class Util:

    def __init__(self):
        pass

    def list_folder_delta_lake(self, dir):
        return mssparkutils.fs.ls(dir)

    def delete_file_delta_lake(self, path):
        mssparkutils.fs.rm(path, True)



import pytest
import json
from delta_lake.DataLake import DataLake
from delta_lake.Util import Util
import delta_lake.const as const


class TestDataLake:

    def __init__(self, spark, storage_account_name, storage_account_access_key, storage_landing, storage_bronze,
                 storage_silver, input_data_param, mount_data_param, colums_validate_merge_param, new_name, database,
                 target, origin):
        self.delta_lake = DataLake(spark, storage_account_name, storage_account_access_key, storage_landing,
                                   storage_bronze, storage_silver, input_data_param, mount_data_param,
                                   colums_validate_merge_param, new_name, database, target, origin)
        self.delta_lake.colums_validate_merge = self.delta_lake.colums_validate_merge_param.split(',')
        self.util = Util()

    def test_initialize_config_storage(self):
        self.delta_lake.initialize_config_storage()
        list_folder = self.util.list_folder_delta_lake("{0}{1}".format(self.delta_lake.storage_landing, const.test))
        assert len(list_folder) != 0

    def test_validate_tables(self):
        self.delta_lake.spark.sql("CREATE TABLE IF NOT EXISTS {0}.{1}".format(self.delta_lake.database,
                                                                              self.delta_lake.new_name))
        self.delta_lake.validate_tables()
        assert self.delta_lake.table_exist == True
        self.delta_lake.spark.sql("DROP TABLE {0}.{1}".format(self.delta_lake.database, self.delta_lake.new_name))

    def test_create_colums_merge(self):
        self.delta_lake.create_colums_merge()
        assert self.delta_lake.string_validate_columns == "{0}.{2} = {1}.{2}".format(self.delta_lake.origin,
                                                                          self.delta_lake.target,
                                                                          self.delta_lake.new_name)




from abc import ABC, abstractmethod
from pyspark.sql.functions import *
import json


class DataLake(ABC):
    events = None
    string_validate_columns = ""
    input_data = ""
    mount_data = ""
    json_load_insert_values = {}
    colums_validate_merge = []
    colums_silver_array = []

    def __init__(self, spark, storage_account_name, storage_account_access_key, storage_landing, storage_bronze,
                 storage_silver, input_data_param, mount_data_param, colums_validate_merge_param, new_name, database, target, origin):
        self.spark = spark
        self.storage_account_name=storage_account_name
        self.storage_account_access_key=storage_account_access_key
        self.storage_landing = storage_landing
        self.storage_bronze = storage_bronze
        self.storage_silver = storage_silver
        self.input_data_param = input_data_param
        self.mount_data_param = mount_data_param
        self.colums_validate_merge_param = colums_validate_merge_param
        self.new_name = new_name
        self.table_name = new_name
        self.table_exist = False
        self.database = database
        self.target = target
        self.origin = origin

    abstractmethod
    def merge(self):
        pass

    abstractmethod
    def initialize_variables(self):
        pass

    abstractmethod
    def load_data(self):
        pass

    def initialize_config_storage(self):
        self.spark.conf.set(
            "fs.azure.account.key." + self.storage_account_name + ".blob.core.windows.net",
            self.storage_account_access_key)

    def validate_tables(self):
        list_tables = self.spark.catalog.listTables(self.database)
        self.table_exist = False
        for item in list_tables:
            print(item)
            print(type(item))
            print(item.name)
            self.table_exist = item.name.lower() == self.table_name.lower()
            if self.table_exist:
                break

    def load_data_csv(self, input_data):
        self.events = self.spark.read.format("csv").option("header", "true").load(input_data)

    def create_colums_merge(self):
        string_and = "and"
        for item in self.colums_validate_merge:
            condition = "{0}.{2} = {1}.{2}".format(self.origin, self.target, item)
            if item is not "":
                if self.string_validate_columns is "":
                    self.string_validate_columns = condition
                else:
                    self.string_validate_columns = "{0} {1} {2}".format(self.string_validate_columns, string_and, condition)
        print(self.string_validate_columns)

    def create_json_columns_pass(self):
        insert_values = "{"
        for item in self.colums_silver_array:
            if item is not "":
                insert_values = "{0}{1}".format(insert_values, '"{0}":"{1}.{0}", '.format(item, self.origin))

        insert_values = insert_values[0: len(insert_values) - 2] + '}'

        self.json_load_insert_values = json.loads(insert_values)
        print(self.json_load_insert_values)

    abstractmethod
    def create_json_columns_pass(self):
        pass


from delta_lake.DataLake import DataLake
from delta.tables import DeltaTable


class Bronze(DataLake):

    def __init__(self, spark, storage_account_name, storage_account_access_key, storage_landing, storage_bronze,
                 storage_silver, input_data_param, mount_data_param, colums_validate_merge_param, new_name, database,
                 target, origin):
        self.storage_bronze = storage_bronze
        super().__init__(spark, storage_account_name, storage_account_access_key, storage_landing, storage_bronze,
                         storage_silver, input_data_param, mount_data_param, colums_validate_merge_param, new_name,
                         database, target, origin)
        self.initialize_variables()

    def merge(self):
        try:
            if self.table_exist:
                deltaTable = DeltaTable.forPath(self.spark, self.mount_data)
                deltaTable.alias(self.target) \
                    .merge(self.events.alias(self.origin), self.string_validate_columns) \
                    .whenMatchedUpdateAll() \
                    .whenNotMatchedInsertAll() \
                    .execute()
            else:
                self.events.write.format("delta").save(self.mount_data)
                self.spark.sql("CREATE TABLE IF NOT EXISTS {0}.{1} USING DELTA LOCATION '{2}'".format(self.database,
                                                                                                      self.table_name,
                                                                                                      self.mount_data))
        except Exception as e:
            print("Error update or create table: " + self.table_name + " explain: " + str(e))

    def initialize_variables(self):
        self.input_data = "{0}{1}".format(self.storage_landing, self.input_data_param)
        self.mount_data = "{0}{1}".format(self.storage_bronze, self.mount_data_param)
        self.colums_validate_merge = self.colums_validate_merge_param.split(',')

    def load_data(self, input_data):
        self.events = self.spark.read.parquet(input_data)

    def create_json_columns_pass(self):
        pass



from setuptools import setup
import os
from setuptools import setup

# with open('requirements.txt') as f:
#     required = f.read().splitlines()

setup(name='test_data_lake',
      version='0.0.0.29',
      description='processes Datalake to Bronze-Silver_Gold',
      url='https://upload.pypi.org/test_alex',
      author='Imparnert',
      author_email='alex-12-04@hotmail.com',
      license='MIT',
      packages=['delta_lake', 'delta_lake/test'],
      # entry_points={'console_scripts': ['Package = Package.__main__:main'], },
      # install_requires=required,
      classifiers=["Programming Language :: Python :: 3"],
      python_requires='>=3',
      zip_safe=False,
      # scripts=[
      #           'requirements.txt',
      #           'README.md',
      #          ],
      long_description=open('README.md').read(),
      # include_package_data=True,
      )
